
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stddef.h>
#include <stdio.h>

#include "logger.h"
#include "pldstr.h"


/*------------------------------------------------------------------------
 Procedure:     PLD_strncpy ID:1
 Purpose:       Copy characters from 'src' to 'dst', writing not more than 'len'
                characters to the destination, including the terminating \0.
                Thus, for any effective copying, len must be > 1.
 Input:         char *dst: Destination string
                char *src: Source string
                size_t len: length of string
 Output:        Returns a pointer to the destination string.
 Errors:
------------------------------------------------------------------------*/
char *PLD_strncpy (char *dst, const char *src, size_t len)
{

	// Thanks go to 'defrost' of #c for providing the replacement
	//		code which you now see here.  It covers the errors better
	//		than my own previous code.

	// If we have no buffer space, then it's futile attempting
	//		to copy anything, just return NULL
 	if (len==0) return NULL;

	// Providing our destination pointer isn't NULL, we can
	//		commence copying data across

	if (dst)
	{
		char *dp = dst;

		// If our source string exists, start moving it to the
		//		destination string character at a time.
		if (src)
		{
			char *sp = (char *)src;
			while ((--len)&&(*sp)) { *dp=*sp; dp++; sp++; }
		}

		*dp='\0';
	}

	return dst;
}



/*------------------------------------------------------------------------
 Procedure:     PLD_strncat ID:1
 Purpose:       Buffer size limited string concat function for two strings.
 Input:         char *dst: Destination string
                char *src: Source string
                size_t len: Destination string buffer size - total string size cannot exceed this
 Output:
 Errors:        If the length of both strings in total is greater than the available buffer space
                in *dst, we copy the maximum possible amount of chars from *src such that
                buffer does not overflow.  A suffixed '\0' will always be appended.
------------------------------------------------------------------------*/
char *PLD_strncat( char *dst, const char *src, size_t len )
{
	char *dp = dst;
	const char *sp = src;
	int cc;

	if (len == 0) return dst;

	len--;

	// Locate the end of the current string.
	cc = 0;
	while ((*dp)&&(cc < len)) { dp++; cc++; }

	// If we have no more buffer space, then return the destination

	if (cc >= len) return dst;

	// While we have more source, and there's more char space left in the buffer

	while ((*sp)&&(cc < len))
	{
		cc++;
		*dp = *sp;
		dp++;
		sp++;
	}

	// Terminate dst, as a gaurantee of string ending.

	*dp = '\0';

	return dst;
}


/*------------------------------------------------------------------------
 Procedure:     PLD_strncate ID:1
 Purpose:       Catencates a source string to the destination string starting from a given
                endpoint.  This allows for faster catencation of strings by avoiding the
                computation required to locate the endpoint of the destination string.
 Input:         char *dst: Destination string
                char *src: Source string
                size_t len: Destination buffer size
                char *endpoint: Endpoint of destination string, location from where new
                string will be appended
 Output:
 Errors:
------------------------------------------------------------------------*/
char *PLD_strncate( char *dst, const char *src, size_t len, char *endpoint )
{
	char *dp = dst;
	const char *sp = src;
	int cc = 0;

	if (len == 0) return dst;

	len--;

	// If endpoint does not relate correctly, then force manual detection
	// of the endpoint.

	if ((!endpoint)||(endpoint == dst)||((endpoint -dst +1)>len))
	{
		// Locate the end of the current string.
		cc = 0;
		while ((*dp != '\0')&&(cc < len)) { dp++; cc++; }
	}
	else {
		cc = endpoint -dst +1;
		dp = endpoint;
	}

	// If we have no more buffer space, then return the destination

	if (cc >= len) return dst;

	// While we have more source, and there's more char space left in the buffer

	while ((*sp)&&(cc < len))
	{
		cc++;
		*dp = *sp;
		dp++;
		sp++;
	}

	// Terminate dst, as a gaurantee of string ending.

	*dp = '\0';

	return dst;
}





/*------------------------------------------------------------------------
Procedure:     XAM_strncasecmp ID:1
Purpose:       Portable version of strncasecmp(), this may be removed in later
versions as the strncase* type functions are more widely
implemented
Input:
Output:
Errors:
------------------------------------------------------------------------*/
int PLD_strncasecmp( char *s1, char *s2, int n )
{
	char *ds1 = s1, *ds2 = s2;
	char c1, c2;
	int result = 0;

	while(n > 0)
	{
		c1 = tolower(*ds1);
		c2 = tolower(*ds2);

		if (c1 == c2)
		{
			n--;
			ds1++;
			ds2++;
		}
		else
		{
			result = c2 - c1;
			n = 0;
		}

	}

	return result;

}





/*------------------------------------------------------------------------
Procedure:     XAM_strtok ID:1
Purpose:       A thread safe version of strtok()
Input:
Output:
Errors:
------------------------------------------------------------------------*/
char *PLD_strtok( struct PLD_strtok *st, char *line, char *delimeters )
{
	char *stop;
	char *dc;
	char *result = NULL;

	if ( line )
	{
		st->start = line;
	}

	//Strip off any leading delimeters

	dc = delimeters;
	while ((st->start)&&(*dc != '\0'))
	{
		if (*dc == *(st->start))
		{
			st->start++;
			dc = delimeters;
		}
		else dc++;
	}

	// Where we are left, is the start of our token.

	result = st->start;

	if ((st->start)&&(st->start != '\0'))
	{
		stop = strpbrk( st->start, delimeters ); /* locate our next delimeter */

		// If we found a delimeter, then that is good.  We must now break the string here
		// and don't forget to store the character which we stopped on.  Very useful bit
		// of information for programs which process expressions.

		if (stop)
		{

			// Store our delimeter.

			st->delimeter = *stop;

			// Terminate our token.

			*stop = '\0';


			// Because we're emulating strtok() behaviour here, we have to
				// absorb all the concurrent delimeters, that is, unless we
				// reach the end of the string, we cannot return a string with
				// no chars.

			stop++;
			dc = delimeters;
			while (*dc != '\0')
			{
				if (*dc == *stop)
				{
					stop++;
					dc = delimeters;
				}
				else dc++;
			} // While

			if (*stop == '\0') st->start = NULL;
			else st->start = stop;

		}
		else {
			st->start = NULL;
			st->delimeter = '\0';
		}
	}
	else  {
		st->start = NULL;
		result = NULL;
	}


	return result;
}



/*------------------------------------------------------------------------
 Procedure:     PLD_strlower ID:1
 Purpose:       Converts a string to lowercase
 Input:         char *convertme : string to convert
 Output:
 Errors:
------------------------------------------------------------------------*/
int PLD_strlower( unsigned char *convertme )
{

	// Updates:
	//	09-11-2002 - changed from 'char *' to 'unsigned char *' to deal with
	//		non-ASCII characters ( ie, french ).  Pointed out by Emmanuel Collignon

	unsigned char *c = convertme;

	while ( *c != '\0') {*c = tolower(*c); c++;}

	return 0;
}





//-----------------END.
